<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Product Export Entity for Magento 2 (System)
 */

namespace Amasty\ProductExportEntity\Export\DataHandling\FieldGenerator;

use Amasty\ExportCore\Api\VirtualField\GeneratorInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Setup\CategorySetup;
use Magento\CatalogUrlRewrite\Model\ProductUrlPathGenerator;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\EntityManager\EntityMetadataInterface;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\Url;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;

class ProductUrl implements GeneratorInterface
{
    /**
     * @var array
     */
    private $productUrlSuffix = [];

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var Url
     */
    private $urlBuilder;

    /**
     * @var EntityMetadataInterface
     */
    private $productMetadata;

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Url $urlBuilder,
        ResourceConnection $resourceConnection,
        MetadataPool $metadataPool
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
        $this->resourceConnection = $resourceConnection;
        $this->productMetadata = $metadataPool->getMetadata(ProductInterface::class);
    }

    public function generateValue(array $currentRecord): string
    {
        $value = '';
        $urlKey = $currentRecord['url_key'] ?? $this->getUrlKey($currentRecord);
        if ($urlKey) {
            $storeId = $currentRecord['store_id'] ?? Store::DEFAULT_STORE_ID;
            $url = $this->urlBuilder->getUrl($urlKey, ['_scope' => $storeId]);
            $value = rtrim($url, '/') . $this->getUrlSuffix((int)$storeId);
        }

        return $value;
    }

    private function getUrlSuffix(int $storeId): string
    {
        if (!isset($this->productUrlSuffix[$storeId])) {
            $this->productUrlSuffix[$storeId] = $this->scopeConfig->getValue(
                ProductUrlPathGenerator::XML_PATH_PRODUCT_URL_SUFFIX,
                ScopeInterface::SCOPE_STORE,
                $storeId
            ) ?? '';
        }

        return $this->productUrlSuffix[$storeId];
    }

    private function getUrlKey(array $currentRecord): string
    {
        $linkField = $this->productMetadata->getLinkField();
        $linkFieldValue = $currentRecord[$linkField] ?? null;
        if (null === $linkFieldValue) {
            return '';
        }

        $varAttrTable = $this->resourceConnection->getTableName('catalog_product_entity_varchar');
        $attrTable = $this->resourceConnection->getTableName('eav_attribute');
        $entityTable = $this->resourceConnection->getTableName('catalog_product_entity');
        $connection = $this->resourceConnection->getConnection();
        $storeId = $currentRecord['store_id'] ?? Store::DEFAULT_STORE_ID;

        $subSelect = $connection->select()
            ->from(['a' => $attrTable], ['attribute_id'])
            ->where('a.entity_type_id = ? ', CategorySetup::CATALOG_PRODUCT_ENTITY_TYPE_ID)
            ->where('a.attribute_code = ? ', 'url_key');

        $select = $connection->select()
            ->from(['va' => $varAttrTable], ['url_key' => 'va.value'])
            ->joinLeft(
                ['e' => $entityTable],
                'va.' . $linkField . ' = e.' . $linkField,
                []
            )
            ->where('va.attribute_id = ? ', $subSelect)
            ->where('va.' . $linkField . ' = ? ', $linkFieldValue)
            ->where('va.store_id = ? ', $storeId);

        return (string)$connection->fetchOne($select);
    }
}
